@@ -6,6 +6,9 @@ from __future__ import absolute_import, division, print_function
 
 import os
 import sys
+from distutils import dist
+from distutils.ccompiler import get_default_compiler
+from distutils.command.config import config
 
 from _cffi_src.utils import (
     build_ffi_for_binding, compiler_type, extra_link_args
@@ -22,6 +25,7 @@ def _get_openssl_libraries(platform):
         )
         if windows_link_legacy_openssl is None:
             # Link against the 1.1.0 names
+            # CRYPTOGRAPHY_OPENSSL_110_OR_GREATER
             libs = ["libssl", "libcrypto"]
         else:
             # Link against the 1.0.2 and lower names
@@ -32,8 +36,11 @@ def _get_openssl_libraries(platform):
         # In some circumstances, the order in which these libs are
         # specified on the linker command-line is significant;
         # libssl must come before libcrypto
-        # (http://marc.info/?l=openssl-users&m=135361825921871)
-        return ["ssl", "crypto"]
+        # (https://marc.info/?l=openssl-users&m=135361825921871)
+        # -lpthread required due to usage of pthread an potential
+        # existance of a static part containing e.g. pthread_atfork
+        # (https://github.com/pyca/cryptography/issues/5084)
+        return ["ssl", "crypto", "pthread"]
 
 
 def _extra_compile_args(platform):
@@ -46,7 +53,16 @@ def _extra_compile_args(platform):
     When we drop support for CRYPTOGRAPHY_OPENSSL_LESS_THAN_110 we can
     revisit this.
     """
-    if platform not in ["win32", "hp-ux11", "sunos5"]:
+    # make sure the compiler used supports the flags to be added
+    is_gcc = False
+    if get_default_compiler() == "unix":
+        d = dist.Distribution()
+        cmd = config(d)
+        cmd._check_compiler()
+        is_gcc = ("gcc" in cmd.compiler.compiler[0] or
+                  "clang" in cmd.compiler.compiler[0])
+    if is_gcc or not (platform in ["win32", "hp-ux11", "sunos5"] or
+                      platform.startswith("aix")):
         return ["-Wconversion", "-Wno-error=sign-conversion"]
     else:
         return []
